require_relative 'security_group'
require_relative 'subnet'
require_relative 'instance'
require_relative 'instance_dsl'

module Awsom
  class Application

    def initialize(vpc_id:, region:)
      @vpc_id = vpc_id
      @region = region
    end

    def run(files)
      files.each do |f|
        apply f
      end
    end

    private

    def security_group(name, &block)
      sg = SecurityGroup.new(name, vpc_id: @vpc_id)
      sg.created(&block)
    end

    def subnet(name, az:, cidr:, &block)
      az = "#{@region}#{az}"
      subnet = Subnet.new(name, cidr: cidr, vpc_id: @vpc_id, az: az)
      id = subnet.created
      puts "subnet #{name} (#{cidr}) #{id}"
      @subnet_id = id
      instance_eval(&block) if block
    ensure
      @subnet_id = nil
    end

    def instance(name, &block)
      instance = Instance.new(name, vpc_id: @vpc_id)
      dsl = InstanceDSL.new(name, vpc_id: @vpc_id, subnet_id: @subnet_id)
      dsl.read(&block)
      setup_proc = dsl.setup_proc
      instance.created(setup_proc) { dsl.run_params }
    end

    def apply(file)
      instance_eval(File.read(file), file)
    end

  end
end
